@@ -20,6 +20,8 @@
 #include "mojito-view.h"
 #include "mojito-view-ginterface.h"
 
+#include <mojito/mojito-core.h>
+#include <mojito/mojito-debug.h>
 #include <mojito/mojito-item.h>
 #include <mojito/mojito-set.h>
 #include <mojito/mojito-utils.h>
@@ -37,6 +39,7 @@ G_DEFINE_TYPE_WITH_CODE (MojitoView, mojito_view, G_TYPE_OBJECT,
 #define REFRESH_TIMEOUT (5 * 60)
 
 struct _MojitoViewPrivate {
+  MojitoCore *core;
   /* List of MojitoService objects */
   GList *services;
   /* The maximum number of items in the view */
@@ -53,6 +56,7 @@ struct _MojitoViewPrivate {
 
 enum {
   PROP_0,
+  PROP_CORE,
   PROP_COUNT,
 };
 
@@ -108,7 +112,7 @@ static MojitoSet *
 munge_items (MojitoView *view)
 {
   MojitoViewPrivate *priv = view->priv;
-  GList *list;
+  GList *list, *l = NULL;
   int count, service_max;
   GHashTable *counts;
   MojitoSet *new;
@@ -116,6 +120,14 @@ munge_items (MojitoView *view)
   /* The magic */
   list = mojito_set_as_list (priv->all_items);
 
+  while (list) {
+    if (!mojito_core_is_item_banned (priv->core, (MojitoItem *)list->data)) {
+      l = g_list_prepend (l, list->data);
+    }
+    list = g_list_delete_link (list, list);
+  }
+  list = l;
+
   list = g_list_sort (list, (GCompareFunc)mojito_item_compare_date_newer);
 
   counts = g_hash_table_new (NULL, NULL);
@@ -188,6 +200,35 @@ remove_service (GObject *object, gpointer user_data)
   return mojito_item_get_service (item) == service;
 }
 
+void
+mojito_view_recalculate (MojitoView *view)
+{
+  MojitoViewPrivate *priv;
+  MojitoSet *old_items, *new_items;
+  MojitoSet *removed_items, *added_items;
+
+  g_return_if_fail (MOJITO_IS_VIEW (view));
+
+  priv = view->priv;
+  if (!priv->running)
+    return;
+
+  old_items = priv->current;
+  new_items = munge_items (view);
+
+  removed_items = mojito_set_difference (old_items, new_items);
+  added_items = mojito_set_difference (new_items, old_items);
+
+  mojito_set_foreach (removed_items, (GFunc)send_removed, view);
+  mojito_set_foreach (added_items, (GFunc)send_added, view);
+
+  mojito_set_unref (removed_items);
+  mojito_set_unref (added_items);
+
+  mojito_set_unref (priv->current);
+  priv->current = new_items;
+}
+
 static void
 service_updated (MojitoService *service, MojitoSet *set, gpointer user_data)
 {
@@ -200,32 +241,14 @@ service_updated (MojitoService *service, MojitoSet *set, gpointer user_data)
     mojito_cache_drop (service);
   }
 
-  if (priv->running) {
-    MojitoSet *old_items, *new_items;
-    MojitoSet *removed_items, *added_items;
-
-    /* Remove all existing items from this service */
-    mojito_set_foreach_remove (priv->all_items, remove_service, service);
-    if (set) {
-      mojito_set_add_from (priv->all_items, set);
-      mojito_set_unref (set);
-    }
-
-    old_items = priv->current;
-    new_items = munge_items (view);
-
-    removed_items = mojito_set_difference (old_items, new_items);
-    added_items = mojito_set_difference (new_items, old_items);
-
-    mojito_set_foreach (removed_items, (GFunc)send_removed, view);
-    mojito_set_foreach (added_items, (GFunc)send_added, view);
-
-    mojito_set_unref (removed_items);
-    mojito_set_unref (added_items);
-
-    mojito_set_unref (priv->current);
-    priv->current = new_items;
+  /* Remove all existing items from this service */
+  mojito_set_foreach_remove (priv->all_items, remove_service, service);
+  if (set) {
+    mojito_set_add_from (priv->all_items, set);
+    mojito_set_unref (set);
   }
+
+  mojito_view_recalculate (view);
 }
 
 /*
@@ -239,6 +262,9 @@ start_refresh (MojitoView *view)
 
   for (l = priv->services; l; l = l->next) {
     MojitoService *service = l->data;
+    MOJITO_DEBUG (VIEWS, "Refreshing %s on view %p",
+                  mojito_service_get_name (service),
+                  view);
     mojito_service_refresh (service);
   }
 
@@ -368,6 +394,9 @@ mojito_view_get_property (GObject    *object,
   MojitoViewPrivate *priv = MOJITO_VIEW (object)->priv;
 
   switch (property_id) {
+  case PROP_CORE:
+    g_value_set_object (value, priv->core);
+    break;
   case PROP_COUNT:
     g_value_set_uint (value, priv->count);
     break;
@@ -385,6 +414,9 @@ mojito_view_set_property (GObject      *object,
   MojitoViewPrivate *priv = MOJITO_VIEW (object)->priv;
 
   switch (property_id) {
+  case PROP_CORE:
+    priv->core = g_value_dup_object (value);
+    break;
   case PROP_COUNT:
     priv->count = g_value_get_uint (value);
     break;
@@ -399,6 +431,11 @@ mojito_view_dispose (GObject *object)
   MojitoView *view = MOJITO_VIEW (object);
   MojitoViewPrivate *priv = view->priv;
 
+  if (priv->core) {
+    g_object_unref (priv->core);
+    priv->core = NULL;
+  }
+
   if (priv->all_items) {
     mojito_set_unref (priv->all_items);
     priv->all_items = NULL;
@@ -454,6 +491,11 @@ mojito_view_class_init (MojitoViewClass *klass)
   object_class->dispose = mojito_view_dispose;
   object_class->finalize = mojito_view_finalize;
 
+  pspec = g_param_spec_object ("core", "core", "The core",
+                               MOJITO_TYPE_CORE,
+                               G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_CORE, pspec);
+
   pspec = g_param_spec_uint ("count", "count", "The number of items",
                              1, G_MAXUINT, 10,
                              G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
@@ -474,9 +516,10 @@ mojito_view_init (MojitoView *self)
 }
 
 MojitoView*
-mojito_view_new (guint count)
+mojito_view_new (MojitoCore *core, guint count)
 {
   return g_object_new (MOJITO_TYPE_VIEW,
+                       "core", core,
                        "count", count,
                        NULL);
 }
@@ -501,7 +544,7 @@ void
 test_view_new (void)
 {
   MojitoView *view;
-  view = mojito_view_new (5);
+  view = mojito_view_new (NULL, 5);
   g_object_unref (view);
 }
 
